package org.histidine.foundation.campaign.events;

import java.util.Map;
import org.apache.log4j.Logger;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BaseOnMessageDeliveryScript;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.comm.CommMessageAPI;
import com.fs.starfarer.api.campaign.comm.MessagePriority;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.events.CampaignEventPlugin;
import com.fs.starfarer.api.campaign.events.CampaignEventTarget;
import com.fs.starfarer.api.fleet.FleetMemberAPI;
import com.fs.starfarer.api.impl.campaign.events.BaseEventPlugin;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.Strings;
import com.fs.starfarer.api.util.Misc;
import exerelin.campaign.PlayerFactionStore;
import exerelin.utilities.ExerelinUtilsFaction;
import java.util.ArrayList;
import java.util.List;
import org.histidine.foundation.scripts.UtilFuncs;


public class FoundationInsuranceEvent extends BaseEventPlugin {
	
	public static Logger log = Global.getLogger(FoundationInsuranceEvent.class);
	protected String lastFactionId = Factions.INDEPENDENT;
	protected float baseInsuranceMult = 0.8f;
	protected float playerLevelFalloffStart = 15;
	protected float playerLevelFalloffEnd = 25;
	protected float paidAmount = 0f;
	protected boolean done = false;
	
	@Override
	public void init(String type, CampaignEventTarget eventTarget) {
		super.init(type, eventTarget);
	}
	
	@Override
	public void reportBattleFinished(CampaignFleetAPI winner, BattleAPI battle)
	{
		if (!battle.isPlayerInvolved()) return;
		if (Global.getSector().getEventManager().isOngoing(null, "exerelin_faction_insurance"))
		{
			String alignedFactionId = PlayerFactionStore.getPlayerFactionId();
			if (!ExerelinUtilsFaction.getFactionMarkets(alignedFactionId).isEmpty())
				return;	// no double insurance
		}
		
		CampaignFleetAPI fleet = Global.getSector().getPlayerFleet();
		
		float value = 0f;
		String stage = "report";
				
		List<FleetMemberAPI> fleetCurrent = fleet.getFleetData().getMembersListCopy();
		for (FleetMemberAPI member : fleet.getFleetData().getSnapshot()) {
			if (!fleetCurrent.contains(member)) {
				value += member.getBaseBuyValue();
			}
		}
		if (value <= 0) return;
		
		float mult = 1;
		int playerLevel = Global.getSector().getPlayerPerson().getStats().getLevel();
		if (playerLevel >= playerLevelFalloffEnd) {
			done = true;
			return;
		}
		mult *= (playerLevelFalloffEnd - playerLevel) / (playerLevelFalloffEnd - playerLevelFalloffStart);
		if (mult > 1) mult = 1;
		
		
		paidAmount = value * baseInsuranceMult * mult;	// FIXME: need config
		
		MarketAPI closestMarket = UtilFuncs.getClosestMarket();
		if (closestMarket != null)
		{
			//lastFactionId = closestMarket.getFactionId();
			Global.getSector().reportEventStage(this, stage, closestMarket.getPrimaryEntity(), MessagePriority.ENSURE_DELIVERY, new BaseOnMessageDeliveryScript() {
				public void beforeDelivery(CommMessageAPI message) {
					Global.getSector().getPlayerFleet().getCargo().getCredits().add(paidAmount);
				}
			});
		}
	}
	
	@Override
	public String getEventName() {
		return ("Ship loss insurance");
	}
	
	@Override
	public CampaignEventPlugin.CampaignEventCategory getEventCategory() {
		return CampaignEventPlugin.CampaignEventCategory.DO_NOT_SHOW_IN_MESSAGE_FILTER;
	}
	
	@Override
	public Map<String, String> getTokenReplacements() {
		Map<String, String> map = super.getTokenReplacements();
		FactionAPI faction = Global.getSector().getFaction(lastFactionId);
		String factionName = faction.getDisplayName();
		String theFactionName = faction.getDisplayNameLongWithArticle();
		map.put("$sender", factionName);
		//map.put("$employer", factionName);
		//map.put("$Employer", Misc.ucFirst(factionName));
		//map.put("$theEmployer", theFactionName);
		//map.put("$TheEmployer", Misc.ucFirst(theFactionName));
		map.put("$paid", Misc.getWithDGS((int)paidAmount) + Strings.C);
		return map;
	}
	
	@Override
	public String[] getHighlights(String stageId) {
		List<String> result = new ArrayList<>();
		addTokensToList(result, "$paid");
		return result.toArray(new String[0]);
	}
	
	@Override
	public String getCurrentImage() {
		FactionAPI myFaction = Global.getSector().getFaction(lastFactionId);
		return myFaction.getLogo();
	}
	
	@Override
	public boolean isDone() {
		return false;
	}
	
	@Override
	public boolean showAllMessagesIfOngoing() {
		return false;
	}
}