package org.histidine.foundation.scripts.world;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.PlanetAPI;
import com.fs.starfarer.api.campaign.SectorAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.ids.Conditions;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.Submarkets;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

public class FoundationSectorGen {
	public static final String FOUNDATION_HQ_ID = "foundation_lawson";
	public static final String FOUNDATION_HQ_NAME = "Lawson's Point";
	
	public SectorEntityToken createFoundationBase(SectorAPI sector)
	{
		SectorEntityToken corvus = sector.getEntityById("corvus");
		SectorEntityToken toOrbit = null;
		float orbitRadius = 5650;	// Nemo's Belt orbit distance
		
		if (corvus != null)
		{
			toOrbit = corvus;
		}
		else {
			WeightedRandomPicker<SectorEntityToken> picker = new WeightedRandomPicker<>();
			Set<StarSystemAPI> populatedSystems = new HashSet<>();
			for (MarketAPI market : sector.getEconomy().getMarketsCopy())
			{
				StarSystemAPI system = market.getStarSystem();
				if (system != null)	populatedSystems.add(system);
			}
			
			for (StarSystemAPI location : sector.getStarSystems())
			{
				float weight = 100;
				if (!populatedSystems.contains(location))
					weight = 0.00001f;
				for (PlanetAPI planet : location.getPlanets())
				{
					if (planet.isStar()) continue;
					if (planet.getMarket() != null) continue;
					picker.add(planet, weight);
				}
			}
			toOrbit = picker.pick();
			orbitRadius = toOrbit.getRadius() + 200;
		}
		
		if (toOrbit == null)
		{
			Global.getLogger(FoundationSectorGen.class).warn("Unable to find suitable location for " + FOUNDATION_HQ_NAME);
			return null;
		}
		else
		{
			Global.getLogger(FoundationSectorGen.class).info("Spawning " + FOUNDATION_HQ_NAME + " around " + toOrbit.getName());
		}
		
		float orbitPeriod = SystemGenUtils.getOrbitalPeriod(toOrbit, orbitRadius);
		SectorEntityToken foundationBase = toOrbit.getContainingLocation().addCustomEntity(FOUNDATION_HQ_ID, FOUNDATION_HQ_NAME, "station_side00", Factions.INDEPENDENT);
		foundationBase.setCircularOrbitPointingDown(toOrbit, 120, orbitRadius, orbitPeriod);
		foundationBase.setInteractionImage("illustrations", "city_from_above");
		foundationBase.setCustomDescriptionId("foundation_station_lawson");
		
		SystemGenUtils.addMarketplace(Factions.INDEPENDENT,  
				foundationBase, 
				new ArrayList<>(Arrays.asList((SectorEntityToken) foundationBase)),  
				FOUNDATION_HQ_NAME,
				4,  
				new ArrayList<>(Arrays.asList(Conditions.POPULATION_4, Conditions.ORE_COMPLEX, Conditions.LIGHT_INDUSTRIAL_COMPLEX,
						Conditions.ORBITAL_STATION, Conditions.STEALTH_MINEFIELDS, Conditions.VOLATILES_DEPOT)),  
				new ArrayList<>(Arrays.asList(Submarkets.SUBMARKET_STORAGE, Submarkets.SUBMARKET_BLACK, Submarkets.SUBMARKET_OPEN)),  
				0.25f
		);
		foundationBase.getMarket().addSubmarket("foundation_submarket");
		
		return foundationBase;
	}
	
	public void generate(SectorAPI sector) {
		Global.getLogger(FoundationSectorGen.class).info("Running sector generation");
		createFoundationBase(sector);
	}
}
